@@ -248,17 +248,21 @@ gtk_print_backend_initialize (void)
 
   if (!initialized)
     {
+      /**
+       * GtkSettings::gtk-print-backends:
+       *
+       * A comma-separated list of print backends to use in the print
+       * dialog. Available print backends depend on the GTK+ installation,
+       * and may include "pdf", "cups" or "lpr".
+       *
+       * Since: 2.10
+       */
       gtk_settings_install_property (g_param_spec_string ("gtk-print-backends",
 							  P_("Default print backend"),
 							  P_("List of the GtkPrintBackend backends to use by default"),
 							  GTK_PRINT_BACKENDS,
 							  GTK_PARAM_READWRITE));
 
-      gtk_settings_install_property (g_param_spec_string ("gtk-print-preview-command",
-							  P_("Default command to run when displaying a print preview"),
-							  P_("Command to run when displaying a print preview"),
-							  GTK_PRINT_PREVIEW_COMMAND,
-							  GTK_PARAM_READWRITE)); 
       initialized = TRUE;
     }
 }
@@ -280,8 +284,10 @@ gtk_print_backend_load_modules (void)
   gtk_print_backend_initialize ();
   
   settings = gtk_settings_get_default ();
-
-  g_object_get (settings, "gtk-print-backends", &setting, NULL);
+  if (settings)
+    g_object_get (settings, "gtk-print-backends", &setting, NULL);
+  else
+    setting = g_strdup (GTK_PRINT_BACKENDS);
 
   backends = g_strsplit (setting, ",", -1);
 
@@ -570,7 +576,7 @@ gtk_print_backend_find_printer (GtkPrintBackend *backend,
 void
 gtk_print_backend_print_stream (GtkPrintBackend        *backend,
                                 GtkPrintJob            *job,
-                                gint                    data_fd,
+                                GIOChannel             *data_io,
                                 GtkPrintJobCompleteFunc callback,
                                 gpointer                user_data,
 				GDestroyNotify          dnotify)
@@ -579,7 +585,7 @@ gtk_print_backend_print_stream (GtkPrintBackend        *backend,
 
   GTK_PRINT_BACKEND_GET_CLASS (backend)->print_stream (backend,
 						       job,
-						       data_fd,
+						       data_io,
 						       callback,
 						       user_data,
 						       dnotify);
