@@ -35,6 +35,7 @@
 
 #include "gtkprintjob.h"
 #include "gtkprinter.h"
+#include "gtkprinter-private.h"
 #include "gtkprintbackend.h"
 #include "gtkalias.h"
 
@@ -46,7 +47,7 @@ struct _GtkPrintJobPrivate
 {
   gchar *title;
 
-  int spool_file_fd;
+  GIOChannel *spool_io;
   cairo_surface_t *surface;
 
   GtkPrintStatus status;
@@ -85,10 +86,11 @@ enum {
 
 enum {
   PROP_0,
-  GTK_PRINT_JOB_PROP_TITLE,
-  GTK_PRINT_JOB_PROP_PRINTER,
-  GTK_PRINT_JOB_PROP_PAGE_SETUP,
-  GTK_PRINT_JOB_PROP_SETTINGS
+  PROP_TITLE,
+  PROP_PRINTER,
+  PROP_PAGE_SETUP,
+  PROP_SETTINGS,
+  PROP_TRACK_PRINT_STATUS
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -108,8 +110,8 @@ gtk_print_job_class_init (GtkPrintJobClass *class)
 
   g_type_class_add_private (class, sizeof (GtkPrintJobPrivate));
 
-  g_object_class_install_property (G_OBJECT_CLASS (class),
-                                   GTK_PRINT_JOB_PROP_TITLE,
+  g_object_class_install_property (object_class,
+                                   PROP_TITLE,
                                    g_param_spec_string ("title",
 						        P_("Title"),
 						        P_("Title of the print job"),
@@ -117,8 +119,8 @@ gtk_print_job_class_init (GtkPrintJobClass *class)
 							GTK_PARAM_READWRITE |
 						        G_PARAM_CONSTRUCT_ONLY));
 
-  g_object_class_install_property (G_OBJECT_CLASS (class),
-                                   GTK_PRINT_JOB_PROP_PRINTER,
+  g_object_class_install_property (object_class,
+                                   PROP_PRINTER,
                                    g_param_spec_object ("printer",
 						        P_("Printer"),
 						        P_("Printer to print the job to"),
@@ -126,8 +128,8 @@ gtk_print_job_class_init (GtkPrintJobClass *class)
 							GTK_PARAM_READWRITE |
 						        G_PARAM_CONSTRUCT_ONLY));
 
-  g_object_class_install_property (G_OBJECT_CLASS (class),
-                                   GTK_PRINT_JOB_PROP_SETTINGS,
+  g_object_class_install_property (object_class,
+                                   PROP_SETTINGS,
                                    g_param_spec_object ("settings",
 						        P_("Settings"),
 						        P_("Printer settings"),
@@ -135,8 +137,8 @@ gtk_print_job_class_init (GtkPrintJobClass *class)
 							GTK_PARAM_READWRITE |
 						        G_PARAM_CONSTRUCT_ONLY));
 
-  g_object_class_install_property (G_OBJECT_CLASS (class),
-                                   GTK_PRINT_JOB_PROP_PAGE_SETUP,
+  g_object_class_install_property (object_class,
+                                   PROP_PAGE_SETUP,
                                    g_param_spec_object ("page-setup",
 						        P_("Page Setup"),
 						        P_("Page Setup"),
@@ -144,6 +146,17 @@ gtk_print_job_class_init (GtkPrintJobClass *class)
 							GTK_PARAM_READWRITE |
 						        G_PARAM_CONSTRUCT_ONLY));
 
+  g_object_class_install_property (object_class,
+				   PROP_TRACK_PRINT_STATUS,
+				   g_param_spec_boolean ("track-print-status",
+							 P_("Track Print Status"),
+							 P_("TRUE if the print job will continue to emit "
+							    "status-changed signals after the print data "
+							    "has been sent to the printer or print server."),
+							 FALSE,
+							 GTK_PARAM_READWRITE));
+  
+
   /**
    * GtkPrintJob::status-changed:
    * @job: the #GtkPrintJob object on which the signal was emitted
@@ -170,7 +183,7 @@ gtk_print_job_init (GtkPrintJob *job)
 
   priv = job->priv = GTK_PRINT_JOB_GET_PRIVATE (job); 
 
-  priv->spool_file_fd = -1;
+  priv->spool_io = NULL;
 
   priv->title = g_strdup ("");
   priv->surface = NULL;
@@ -231,10 +244,10 @@ gtk_print_job_finalize (GObject *object)
   GtkPrintJob *job = GTK_PRINT_JOB (object);
   GtkPrintJobPrivate *priv = job->priv;
 
-  if (priv->spool_file_fd >= 0)
+  if (priv->spool_io != NULL)
     {
-      close (priv->spool_file_fd);
-      priv->spool_file_fd = -1;
+      g_io_channel_unref (priv->spool_io);
+      priv->spool_io = NULL;
     }
   
   if (priv->backend)
@@ -388,7 +401,8 @@ gtk_print_job_set_status (GtkPrintJob   *job,
  * Make the #GtkPrintJob send an existing document to the 
  * printing system. The file can be in any format understood
  * by the platforms printing system (typically PostScript,
- * but on many platforms PDF may work too).
+ * but on many platforms PDF may work too). See 
+ * gtk_printer_accepts_pdf() and gtk_printer_accepts_ps().
  * 
  * Returns: %FALSE if an error occurred
  *
@@ -400,29 +414,26 @@ gtk_print_job_set_source_file (GtkPrintJob *job,
 			       GError     **error)
 {
   GtkPrintJobPrivate *priv;
+  GError *tmp_error;
+
+  tmp_error = NULL;
 
   g_return_val_if_fail (GTK_IS_PRINT_JOB (job), FALSE);
 
   priv = job->priv;
 
-  priv->spool_file_fd = g_open (filename, O_RDONLY|O_BINARY);
-  if (priv->spool_file_fd < 0)
-    {
-      gchar *display_filename = g_filename_display_name (filename);
-      int save_errno = errno;
-      
-      g_set_error (error,
-                   G_FILE_ERROR,
-                   g_file_error_from_errno (save_errno),
-                   _("Failed to open file '%s': %s"),
-                   display_filename, 
-		   g_strerror (save_errno));
-      
-      g_free (display_filename);
+  priv->spool_io = g_io_channel_new_file (filename, "r", &tmp_error);
 
+  if (tmp_error == NULL)
+    g_io_channel_set_encoding (priv->spool_io, NULL, &tmp_error);
+
+  if (tmp_error != NULL)
+    {
+      g_propagate_error (error, tmp_error);
       return FALSE;
     }
-    return TRUE;
+
+  return TRUE;
 }
 
 /**
@@ -445,7 +456,11 @@ gtk_print_job_get_surface (GtkPrintJob  *job,
   gchar *filename;
   gdouble width, height;
   GtkPaperSize *paper_size;
-  
+  int fd;
+  GError *tmp_error;
+
+  tmp_error = NULL;
+
   g_return_val_if_fail (GTK_IS_PRINT_JOB (job), NULL);
 
   priv = job->priv;
@@ -453,25 +468,46 @@ gtk_print_job_get_surface (GtkPrintJob  *job,
   if (priv->surface)
     return priv->surface;
  
-  g_return_val_if_fail (priv->spool_file_fd == -1, NULL);
+  g_return_val_if_fail (priv->spool_io == NULL, NULL);
  
-  priv->spool_file_fd = g_file_open_tmp ("gtkprint_XXXXXX", 
-						    &filename, 
-						    error);
-  if (priv->spool_file_fd == -1)
-    return NULL;
+  fd = g_file_open_tmp ("gtkprint_XXXXXX", 
+			 &filename, 
+			 &tmp_error);
+  if (fd == -1)
+    {
+      g_propagate_error (error, tmp_error);
+      return NULL;
+    }
 
-  fchmod (priv->spool_file_fd, S_IRUSR | S_IWUSR);
+  fchmod (fd, S_IRUSR | S_IWUSR);
+  
+#ifdef G_ENABLE_DEBUG 
+  /* If we are debugging printing don't delete the tmp files */
+  if (!(gtk_debug_flags & GTK_DEBUG_PRINTING))
+#endif /* G_ENABLE_DEBUG */
   unlink (filename);
 
   paper_size = gtk_page_setup_get_paper_size (priv->page_setup);
   width = gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS);
   height = gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS);
-  
-  priv->surface = _gtk_printer_create_cairo_surface (priv->printer,
-							  width, height,
-							  priv->spool_file_fd);
  
+  priv->spool_io = g_io_channel_unix_new (fd);
+  g_io_channel_set_close_on_unref (priv->spool_io, TRUE);
+  g_io_channel_set_encoding (priv->spool_io, NULL, &tmp_error);
+  
+  if (tmp_error != NULL)
+    {
+      g_io_channel_unref (priv->spool_io);
+      priv->spool_io = NULL;
+      g_propagate_error (error, tmp_error);
+      return NULL;
+    }
+
+  priv->surface = _gtk_printer_create_cairo_surface (priv->printer,
+						     priv->settings,
+						     width, height,
+						     priv->spool_io);
+  
   return priv->surface;
 }
 
@@ -492,7 +528,7 @@ gtk_print_job_get_surface (GtkPrintJob  *job,
  */
 void
 gtk_print_job_set_track_print_status (GtkPrintJob *job,
-				      gboolean track_status)
+				      gboolean     track_status)
 {
   GtkPrintJobPrivate *priv;
 
@@ -500,7 +536,14 @@ gtk_print_job_set_track_print_status (GtkPrintJob *job,
 
   priv = job->priv;
 
-  priv->track_print_status = track_status;
+  track_status = track_status != FALSE;
+
+  if (priv->track_print_status != track_status)
+    {
+      priv->track_print_status = track_status;
+      
+      g_object_notify (G_OBJECT (job), "track-print-status");
+    }
 }
 
 /**
@@ -539,22 +582,22 @@ gtk_print_job_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case GTK_PRINT_JOB_PROP_TITLE:
+    case PROP_TITLE:
       priv->title = g_value_dup_string (value);
       break;
     
-    case GTK_PRINT_JOB_PROP_PRINTER:
+    case PROP_PRINTER:
       priv->printer = GTK_PRINTER (g_value_dup_object (value));
       priv->printer_set = TRUE;
       priv->backend = g_object_ref (gtk_printer_get_backend (priv->printer));
       break;
 
-    case GTK_PRINT_JOB_PROP_PAGE_SETUP:
+    case PROP_PAGE_SETUP:
       priv->page_setup = GTK_PAGE_SETUP (g_value_dup_object (value));
       priv->page_setup_set = TRUE;
       break;
       
-    case GTK_PRINT_JOB_PROP_SETTINGS:
+    case PROP_SETTINGS:
       /* We save a copy of the settings since we modify
        * if when preparing the printer job. */
       settings = GTK_PRINT_SETTINGS (g_value_get_object (value));
@@ -562,6 +605,10 @@ gtk_print_job_set_property (GObject      *object,
       priv->settings_set = TRUE;
       break;
 
+    case PROP_TRACK_PRINT_STATUS:
+      gtk_print_job_set_track_print_status (job, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -579,18 +626,21 @@ gtk_print_job_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case GTK_PRINT_JOB_PROP_TITLE:
+    case PROP_TITLE:
       g_value_set_string (value, priv->title);
       break;
-    case GTK_PRINT_JOB_PROP_PRINTER:
+    case PROP_PRINTER:
       g_value_set_object (value, priv->printer);
       break;
-    case GTK_PRINT_JOB_PROP_SETTINGS:
+    case PROP_SETTINGS:
       g_value_set_object (value, priv->settings);
       break;
-    case GTK_PRINT_JOB_PROP_PAGE_SETUP:
+    case PROP_PAGE_SETUP:
       g_value_set_object (value, priv->page_setup);
       break;
+    case PROP_TRACK_PRINT_STATUS:
+      g_value_set_boolean (value, priv->track_print_status);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -600,42 +650,58 @@ gtk_print_job_get_property (GObject    *object,
 /**
  * gtk_print_job_send:
  * @job: a GtkPrintJob
- * @callback: function to call when the job completes
+ * @callback: function to call when the job completes or an error occures
  * @user_data: user data that gets passed to @callback
  * @dnotify: destroy notify for @user_data
- * @error: return location for errors, or %NULL
  * 
  * Sends the print job off to the printer.  
  * 
- * Return value: %FALSE if an error occurred
- *
  * Since: 2.10
  **/
-gboolean
+void
 gtk_print_job_send (GtkPrintJob             *job,
                     GtkPrintJobCompleteFunc  callback,
                     gpointer                 user_data,
-		    GDestroyNotify           dnotify,
-		    GError                 **error)
+		    GDestroyNotify           dnotify)
 {
   GtkPrintJobPrivate *priv;
 
-  g_return_val_if_fail (GTK_IS_PRINT_JOB (job), FALSE);
+  g_return_if_fail (GTK_IS_PRINT_JOB (job));
 
   priv = job->priv;
-  g_return_val_if_fail (priv->spool_file_fd > 0, FALSE);
+  g_return_if_fail (priv->spool_io != NULL);
   
   gtk_print_job_set_status (job, GTK_PRINT_STATUS_SENDING_DATA);
-  lseek (priv->spool_file_fd, 0, SEEK_SET);
-  gtk_print_backend_print_stream (priv->backend,
-                                  job,
-				  priv->spool_file_fd,
-                                  callback,
-                                  user_data,
-				  dnotify);
-
-  return TRUE;
+  
+  g_io_channel_seek_position (priv->spool_io, 0, G_SEEK_SET, NULL);
+  
+  gtk_print_backend_print_stream (priv->backend, job,
+				  priv->spool_io,
+                                  callback, user_data, dnotify);
 }
 
+GType
+gtk_print_capabilities_get_type (void)
+{
+  static GType etype = 0;
+
+  if (etype == 0)
+    {
+      static const GFlagsValue values[] = {
+        { GTK_PRINT_CAPABILITY_PAGE_SET, "GTK_PRINT_CAPABILITY_PAGE_SET", "page-set" },
+        { GTK_PRINT_CAPABILITY_COPIES, "GTK_PRINT_CAPABILITY_COPIES", "copies" },
+        { GTK_PRINT_CAPABILITY_COLLATE, "GTK_PRINT_CAPABILITY_COLLATE", "collate" },
+        { GTK_PRINT_CAPABILITY_REVERSE, "GTK_PRINT_CAPABILITY_REVERSE", "reverse" },
+        { GTK_PRINT_CAPABILITY_SCALE, "GTK_PRINT_CAPABILITY_SCALE", "scale" },
+        { 0, NULL, NULL }
+      };
+
+      etype = g_flags_register_static (I_("GtkPrintCapabilities"), values);
+    }
+
+  return etype;
+}
+
+
 #define __GTK_PRINT_JOB_C__
 #include "gtkaliasdef.c"
