@@ -36,7 +36,9 @@
 
 #define GTK_PRINT_OPERATION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_PRINT_OPERATION, GtkPrintOperationPrivate))
 
-enum {
+enum 
+{
+  DONE,
   BEGIN_PRINT,
   PAGINATE,
   REQUEST_PAGE_SETUP,
@@ -49,7 +51,8 @@ enum {
   LAST_SIGNAL
 };
 
-enum {
+enum 
+{
   PROP_0,
   PROP_DEFAULT_PAGE_SETUP,
   PROP_PRINT_SETTINGS,
@@ -59,10 +62,9 @@ enum {
   PROP_USE_FULL_PAGE,
   PROP_TRACK_PRINT_STATUS,
   PROP_UNIT,
-  PROP_SHOW_DIALOG,
-  PROP_SHOW_PREVIEW,
   PROP_SHOW_PROGRESS,
-  PROP_PDF_TARGET,
+  PROP_ALLOW_ASYNC,
+  PROP_EXPORT_FILENAME,
   PROP_STATUS,
   PROP_STATUS_STRING,
   PROP_CUSTOM_TAB_LABEL
@@ -118,7 +120,7 @@ gtk_print_operation_finalize (GObject *object)
   if (priv->print_settings)
     g_object_unref (priv->print_settings);
   
-  g_free (priv->pdf_target);
+  g_free (priv->export_filename);
   g_free (priv->job_name);
   g_free (priv->custom_tab_label);
 
@@ -128,6 +130,9 @@ gtk_print_operation_finalize (GObject *object)
   if (priv->show_progress_timeout_id > 0)
     g_source_remove (priv->show_progress_timeout_id);
 
+  if (priv->error)
+    g_error_free (priv->error);
+  
   G_OBJECT_CLASS (gtk_print_operation_parent_class)->finalize (object);
 }
 
@@ -146,10 +151,8 @@ gtk_print_operation_init (GtkPrintOperation *operation)
   priv->nr_of_pages = -1;
   priv->current_page = -1;
   priv->use_full_page = FALSE;
-  priv->show_dialog = TRUE;
-  priv->show_preview = FALSE;
   priv->show_progress = FALSE;
-  priv->pdf_target = NULL;
+  priv->export_filename = NULL;
   priv->track_print_status = FALSE;
   priv->is_sync = FALSE;
 
@@ -162,8 +165,9 @@ gtk_print_operation_init (GtkPrintOperation *operation)
 
 static void
 preview_iface_render_page (GtkPrintOperationPreview *preview,
-			   gint page_nr)
+			   gint                      page_nr)
 {
+
   GtkPrintOperation *op;
 
   op = GTK_PRINT_OPERATION (preview);
@@ -183,6 +187,9 @@ preview_iface_end_preview (GtkPrintOperationPreview *preview)
     g_main_loop_quit (op->priv->rloop);
 
   op->priv->end_run (op, op->priv->is_sync, TRUE);
+  
+  g_signal_emit (op, signals[DONE], 0,
+		 GTK_PRINT_OPERATION_RESULT_APPLY);
 }
 
 static gboolean
@@ -227,7 +234,7 @@ preview_start_page (GtkPrintOperation *op,
 		    GtkPrintContext   *print_context,
 		    GtkPageSetup      *page_setup)
 {
-  g_signal_emit_by_name (op, "got-page-size",print_context, page_setup);
+  g_signal_emit_by_name (op, "got-page-size", print_context, page_setup);
 }
 
 static void
@@ -282,17 +289,14 @@ gtk_print_operation_set_property (GObject      *object,
     case PROP_UNIT:
       gtk_print_operation_set_unit (op, g_value_get_enum (value));
       break;
-    case PROP_SHOW_DIALOG:
-      gtk_print_operation_set_show_dialog (op, g_value_get_boolean (value));
-      break;
-    case PROP_SHOW_PREVIEW:
-      gtk_print_operation_set_show_preview (op, g_value_get_boolean (value));
+    case PROP_ALLOW_ASYNC:
+      gtk_print_operation_set_allow_async (op, g_value_get_boolean (value));
       break;
     case PROP_SHOW_PROGRESS:
       gtk_print_operation_set_show_progress (op, g_value_get_boolean (value));
       break;
-    case PROP_PDF_TARGET:
-      gtk_print_operation_set_pdf_target (op, g_value_get_string (value));
+    case PROP_EXPORT_FILENAME:
+      gtk_print_operation_set_export_filename (op, g_value_get_string (value));
       break;
     case PROP_CUSTOM_TAB_LABEL:
       gtk_print_operation_set_custom_tab_label (op, g_value_get_string (value));
@@ -338,17 +342,14 @@ gtk_print_operation_get_property (GObject    *object,
     case PROP_UNIT:
       g_value_set_enum (value, priv->unit);
       break;
-    case PROP_SHOW_DIALOG:
-      g_value_set_boolean (value, priv->show_dialog);
-      break;
-    case PROP_SHOW_PREVIEW:
-      g_value_set_boolean (value, priv->show_preview);
+    case PROP_ALLOW_ASYNC:
+      g_value_set_boolean (value, priv->allow_async);
       break;
     case PROP_SHOW_PROGRESS:
       g_value_set_boolean (value, priv->show_progress);
       break;
-    case PROP_PDF_TARGET:
-      g_value_set_string (value, priv->pdf_target);
+    case PROP_EXPORT_FILENAME:
+      g_value_set_string (value, priv->export_filename);
       break;
     case PROP_STATUS:
       g_value_set_enum (value, priv->status);
@@ -386,13 +387,14 @@ preview_print_idle_done (gpointer data)
   
   op = GTK_PRINT_OPERATION (pop->preview);
 
+  cairo_surface_finish (pop->surface);
+  /* Surface is destroyed in launch_preview */
   _gtk_print_operation_platform_backend_launch_preview (op,
+							pop->surface,
 							pop->parent,
 							pop->filename);
 
   g_free (pop->filename);
-  cairo_surface_finish (pop->surface);
-  cairo_surface_destroy (pop->surface);
 
   gtk_print_operation_preview_end_preview (pop->preview);
   g_free (pop);
@@ -416,7 +418,7 @@ preview_print_idle (gpointer data)
   gtk_print_operation_preview_render_page (pop->preview, pop->page_nr);
   
   cr = gtk_print_context_get_cairo_context (pop->print_context);
-  cairo_show_page (cr);
+  _gtk_print_operation_platform_backend_preview_end_page (op, pop->surface, cr);
   
   /* TODO: print out sheets not pages and follow ranges */
   pop->page_nr++;
@@ -432,62 +434,46 @@ static void
 preview_got_page_size (GtkPrintOperationPreview *preview, 
 		       GtkPrintContext          *context,
 		       GtkPageSetup             *page_setup,
-		       PreviewOp *pop)
+		       PreviewOp                *pop)
 {
   GtkPrintOperation *op = GTK_PRINT_OPERATION (preview);
+  cairo_t *cr;
 
   _gtk_print_operation_platform_backend_resize_preview_surface (op, page_setup, pop->surface);
+
+  cr = gtk_print_context_get_cairo_context (pop->print_context);
+  _gtk_print_operation_platform_backend_preview_start_page (op, pop->surface, cr);
+
 }
 
 static void
 preview_ready (GtkPrintOperationPreview *preview,
-               GtkPrintContext *context,
-	       PreviewOp *pop)
+               GtkPrintContext          *context,
+	       PreviewOp                *pop)
 {
-
   pop->page_nr = 0;
   pop->print_context = context;
 
   g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
 	           preview_print_idle,
 		   pop,
-		   preview_print_idle_done); 
-
+		   preview_print_idle_done);
 }
 
-/**
- * gtk_print_operation_preview_handler:
- *
- * Default handler for preview operations
- **/
+
 static gboolean
-gtk_print_operation_preview_handler (GtkPrintOperation *op,
+gtk_print_operation_preview_handler (GtkPrintOperation        *op,
                                      GtkPrintOperationPreview *preview, 
 				     GtkPrintContext          *context,
 				     GtkWindow                *parent)
 {
   gdouble dpi_x, dpi_y;
-  gchar *tmp_dir;
-  gchar *dir_template;
-  gchar *preview_filename;
   PreviewOp *pop;
   GtkPageSetup *page_setup;
   cairo_t *cr;
 
-  dir_template = g_build_filename (g_get_tmp_dir (), "print-preview-XXXXXX", NULL);
-
-  /* use temp dirs because apps like evince need to have extentions
-     to determine the mine type */
-  tmp_dir = mkdtemp(dir_template);
-
-  preview_filename = g_build_filename (tmp_dir, 
-                                      "Print Preview.pdf",
-                                      NULL);
-  
-  g_free (dir_template);
-
   pop = g_new0 (PreviewOp, 1);
-  pop->filename = preview_filename;
+  pop->filename = NULL;
   pop->preview = preview;
   pop->parent = parent;
 
@@ -497,7 +483,7 @@ gtk_print_operation_preview_handler (GtkPrintOperation *op,
     _gtk_print_operation_platform_backend_create_preview_surface (op,
 								  page_setup,
 								  &dpi_x, &dpi_y,
-								  pop->filename);
+								  &pop->filename);
 
   cr = cairo_create (pop->surface);
   gtk_print_context_set_cairo_context (op->priv->print_context, cr,
@@ -525,9 +511,9 @@ custom_widget_accumulator (GSignalInvocationHint *ihint,
   gboolean continue_emission;
   GtkWidget *widget;
   
-  widget = g_value_get_pointer (handler_return);
+  widget = g_value_get_object (handler_return);
   if (widget != NULL)
-    g_value_set_pointer (return_accu, widget);
+    g_value_set_object (return_accu, widget);
   continue_emission = (widget == NULL);
   
   return continue_emission;
@@ -547,12 +533,38 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
   
   g_type_class_add_private (gobject_class, sizeof (GtkPrintOperationPrivate));
 
+  /**
+   * GtkPrintOperation::done:
+   * @operation: the #GtkPrintOperation on which the signal was emitted
+   * @result: the result of the print operation
+   *
+   * Emitted when the print operation run has finished doing
+   * everything required for printing. @result gives you information
+   * about what happened during the run. If @result is
+   * %GTK_PRINT_OPERATION_RESULT_ERROR then you can call
+   * gtk_print_operation_get_error() for more information.
+   *
+   * If you enabled print status tracking then 
+   * gtk_print_operation_is_finished() may still return %FALSE 
+   * after this was emitted.
+   *
+   * Since: 2.10
+   */
+  signals[DONE] =
+    g_signal_new (I_("done"),
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkPrintOperationClass, done),
+		  NULL, NULL,
+		  g_cclosure_marshal_VOID__ENUM,
+		  G_TYPE_NONE, 1, GTK_TYPE_PRINT_OPERATION_RESULT);
+
   /**
    * GtkPrintOperation::begin-print:
    * @operation: the #GtkPrintOperation on which the signal was emitted
    * @context: the #GtkPrintContext for the current operation
    *
-   * Gets emitted after the user has finished changing print settings
+   * Emitted after the user has finished changing print settings
    * in the dialog, before the actual rendering starts. 
    *
    * A typical use for this signal is to use the parameters from the
@@ -575,7 +587,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * @operation: the #GtkPrintOperation on which the signal was emitted
    * @context: the #GtkPrintContext for the current operation
    *
-   * Gets emitted after the begin-print signal, but before the actual 
+   * Emitted after the begin-print signal, but before the actual 
    * rendering starts. It keeps getting emitted until it returns %FALSE. 
    *
    * This signal is intended to be used for paginating the document
@@ -607,7 +619,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * @page_nr: the number of the currently printed page
    * @setup: the #GtkPageSetup 
    * 
-   * Gets emitted once for every page that is printed, to give
+   * Emitted once for every page that is printed, to give
    * the application a chance to modify the page setup. Any changes 
    * done to @setup will be in force only for printing this page.
    *
@@ -631,7 +643,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * @context: the #GtkPrintContext for the current operation
    * @page_nr: the number of the currently printed page
    *
-   * Gets emitted for every page that is printed. The signal handler
+   * Emitted for every page that is printed. The signal handler
    * must render the @page_nr's page onto the cairo context obtained
    * from @context using gtk_print_context_get_cairo_context().
    *
@@ -699,7 +711,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * @operation: the #GtkPrintOperation on which the signal was emitted
    * @context: the #GtkPrintContext for the current operation
    *
-   * Gets emitted after all pages have been rendered. 
+   * Emitted after all pages have been rendered. 
    * A handler for this signal can clean up any resources that have
    * been allocated in the ::begin-print handler.
    * 
@@ -718,7 +730,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * GtkPrintOperation::status-changed:
    * @operation: the #GtkPrintOperation on which the signal was emitted
    *
-   * Gets emitted at between the various phases of the print operation.
+   * Emitted at between the various phases of the print operation.
    * See #GtkPrintStatus for the phases that are being discriminated.
    * Use gtk_print_operation_get_status() to find out the current
    * status.
@@ -739,7 +751,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * GtkPrintOperation::create-custom-widget:
    * @operation: the #GtkPrintOperation on which the signal was emitted
    *
-   * Gets emitted when displaying the print dialog. If you return a
+   * Emitted when displaying the print dialog. If you return a
    * widget in a handler for this signal it will be added to a custom
    * tab in the print dialog. You typically return a container widget
    * with multiple widgets in it.
@@ -761,15 +773,15 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkPrintOperationClass, create_custom_widget),
 		  custom_widget_accumulator, NULL,
-		  _gtk_marshal_POINTER__VOID,
-		  G_TYPE_POINTER, 0);
+		  _gtk_marshal_OBJECT__VOID,
+		  G_TYPE_OBJECT, 0);
 
   /**
    * GtkPrintOperation::custom-widget-apply:
    * @operation: the #GtkPrintOperation on which the signal was emitted
    * @widget: the custom widget added in create-custom-widget
    *
-   * This signal gets emitted right before begin-print if you added
+   * Emitted right before begin-print if you added
    * a custom widget in the create-custom-widget handler. When you get
    * this signal you should read the information from the custom widgets,
    * as the widgets are not guaraneed to be around at a later time.
@@ -790,7 +802,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * @operation: the #GtkPrintOperation on which the signal was emitted
    * @preview: the #GtkPrintPreviewOperation for the current operation
    * @context: the #GtkPrintContext that will be used
-   * @parent: the #GtkWindow to use as window parent, or NULL
+   * @parent: the #GtkWindow to use as window parent, or %NULL
    *
    * Gets emitted when a preview is requested from the native dialog.
    * If you handle this you must set the cairo context on the printing context.
@@ -798,7 +810,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * If you don't override this a default implementation using an external
    * viewer will be used.
    *
-   * Returns: #TRUE if the listener wants to take over control of the preview
+   * Returns: %TRUE if the listener wants to take over control of the preview
    * 
    * Since: 2.10
    */
@@ -954,7 +966,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
-				   PROP_USE_FULL_PAGE,
+				   PROP_TRACK_PRINT_STATUS,
 				   g_param_spec_boolean ("track-print-status",
 							 P_("Track Print Status"),
 							 P_("TRUE if the print operation will continue to report on the print job status after the print data has been sent to the printer or print server."),
@@ -980,38 +992,6 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 						      GTK_UNIT_PIXEL,
 						      GTK_PARAM_READWRITE));
   
-
-  /**
-   * GtkPrintOperation:show-dialog:
-   *
-   * Determines whether calling gtk_print_operation_run() will present
-   * a print dialog to the user, or just print to the default printer.
-   *
-   * Since: 2.10
-   */
-  g_object_class_install_property (gobject_class,
-				   PROP_SHOW_DIALOG,
-				   g_param_spec_boolean ("show-dialog",
-							 P_("Show Dialog"),
-							 P_("TRUE if gtk_print_operation_run() should show the print dialog."),
-							 TRUE,
-							 GTK_PARAM_READWRITE));
-
-  /**
-   * GtkPrintOperation:show-preview:
-   *
-   * Determines whether calling gtk_print_operation_run() will present
-   * the print preview to the user.
-   *
-   * Since: 2.10
-   */
-  g_object_class_install_property (gobject_class,
-				   PROP_SHOW_PREVIEW,
-				   g_param_spec_boolean ("show-preview",
-							 P_("Show Preview"),
-							 P_("TRUE if gtk_print_operation_run() should show the print preview."),
-							 TRUE,
-							 GTK_PARAM_READWRITE));
   
   /**
    * GtkPrintOperation:show-progress:
@@ -1028,13 +1008,31 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
 							 P_("TRUE if a progress dialog is shown while printing."),
 							 FALSE,
 							 GTK_PARAM_READWRITE));
-  
+
+  /**
+   * GtkPrintOperation:allow-async:
+   *
+   * Determines whether the print operation may run asynchronously or not.
+   * Some systems don't support asynchronous printing, but those that do
+   * will return %GTK_PRINT_OPERATION_RESULT_IN_PROGRESS as the status, and
+   * emit the done signal when the operation is actually done.
+   *
+   * Since: 2.10
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_ALLOW_ASYNC,
+				   g_param_spec_boolean ("allow-async",
+							 P_("Allow Async"),
+							 P_("TRUE if print process may run asynchronous."),
+							 FALSE,
+							 GTK_PARAM_READWRITE));
   
   /**
-   * GtkPrintOperation:pdf-target:
+   * GtkPrintOperation:export-filename:
    *
-   * The name of a PDF file to generate instead of showing 
-   * the print dialog. 
+   * The name of a file file to generate instead of showing 
+   * the print dialog. Currently, PDF is the only supported
+   * format.
    *
    * The intended use of this property is for implementing 
    * "Export to PDF" actions.
@@ -1046,10 +1044,10 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Since: 2.10
    */
   g_object_class_install_property (gobject_class,
-				   PROP_JOB_NAME,
-				   g_param_spec_string ("pdf-target",
-							P_("PDF target filename"),
-							P_("PDF target filename"),
+				   PROP_EXPORT_FILENAME,
+				   g_param_spec_string ("export-filename",
+							P_("Export filename"),
+							P_("Export filename"),
 							NULL,
 							GTK_PARAM_READWRITE));
   
@@ -1548,6 +1546,10 @@ gtk_print_operation_get_status_string (GtkPrintOperation *op)
  * is finished, either successfully (%GTK_PRINT_STATUS_FINISHED)
  * or unsuccessfully (%GTK_PRINT_STATUS_FINISHED_ABORTED).
  * 
+ * Note: when you enable print status tracking the print operation
+ * can be in a non-finished state even after done has been called, as
+ * the operation status then tracks the print job status on the printer.
+ * 
  * Return value: %TRUE, if the print operation is finished.
  *
  * Since: 2.10
@@ -1565,67 +1567,6 @@ gtk_print_operation_is_finished (GtkPrintOperation *op)
     priv->status == GTK_PRINT_STATUS_FINISHED;
 }
 
-
-/**
- * gtk_print_operation_set_show_dialog:
- * @op: a #GtkPrintOperation
- * @show_dialog: %TRUE to show the print dialog
- * 
- * Sets whether calling gtk_print_operation_run() will present
- * a print dialog to the user, or just print to the default printer.
- *
- * Since: 2.10
- */
-void
-gtk_print_operation_set_show_dialog (GtkPrintOperation *op,
-				     gboolean           show_dialog)
-{
-  GtkPrintOperationPrivate *priv;
-
-  g_return_if_fail (GTK_IS_PRINT_OPERATION (op));
-
-  priv = op->priv;
-
-  show_dialog = show_dialog != FALSE;
-
-  if (priv->show_dialog != show_dialog)
-    {
-      priv->show_dialog = show_dialog;
-
-      g_object_notify (G_OBJECT (op), "show-dialog");
-    }
-}
-
-/**
- * gtk_print_operation_set_show_preview:
- * @op: a #GtkPrintOperation
- * @show_preview: %TRUE to show the print preview
- * 
- * Sets whether calling gtk_print_operation_run() will present
- * the print preview to the user.
- *
- * Since: 2.10
- */
-void
-gtk_print_operation_set_show_preview (GtkPrintOperation *op,
-				      gboolean           show_preview)
-{
-  GtkPrintOperationPrivate *priv;
-
-  g_return_if_fail (GTK_IS_PRINT_OPERATION (op));
-
-  priv = op->priv;
-
-  show_preview = show_preview != FALSE;
-
-  if (priv->show_preview != show_preview)
-    {
-      priv->show_preview = show_preview;
-
-      g_object_notify (G_OBJECT (op), "show-preview");
-    }
-}
-
 /**
  * gtk_print_operation_set_show_progress:
  * @op: a #GtkPrintOperation
@@ -1656,6 +1597,38 @@ gtk_print_operation_set_show_progress (GtkPrintOperation  *op,
     }
 }
 
+/**
+ * gtk_print_operation_set_allow_async:
+ * @op: a #GtkPrintOperation
+ * @allow_async: %TRUE to allow asynchronous operation
+ *
+ * Sets whether the gtk_print_operation_run() may return
+ * before the print operation is completed. Note that
+ * some platforms may not allow asynchronous operation.
+ *
+ * Since: 2.10
+ */
+void
+gtk_print_operation_set_allow_async (GtkPrintOperation  *op,
+				     gboolean            allow_async)
+{
+  GtkPrintOperationPrivate *priv;
+
+  g_return_if_fail (GTK_IS_PRINT_OPERATION (op));
+
+  priv = op->priv;
+
+  allow_async = allow_async != FALSE;
+
+  if (priv->allow_async != allow_async)
+    {
+      priv->allow_async = allow_async;
+
+      g_object_notify (G_OBJECT (op), "allow-async");
+    }
+}
+
+
 /**
  * gtk_print_operation_set_custom_tag_label:
  * @op: a #GtkPrintOperation
@@ -1683,13 +1656,14 @@ gtk_print_operation_set_custom_tab_label (GtkPrintOperation  *op,
 
 
 /**
- * gtk_print_operation_set_pdf_target:
+ * gtk_print_operation_set_export_filename:
  * @op: a #GtkPrintOperation
- * @filename: the filename for the PDF file
+ * @filename: the filename for the exported file
  * 
- * Sets up the #GtkPrintOperation to generate a PDF file instead
+ * Sets up the #GtkPrintOperation to generate a file instead
  * of showing the print dialog. The indended use of this function
- * is for implementing "Export to PDF" actions.
+ * is for implementing "Export to PDF" actions. Currently, PDF
+ * is the only supported format.
  *
  * "Print to PDF" support is independent of this and is done
  * by letting the user pick the "Print to PDF" item from the list
@@ -1698,8 +1672,8 @@ gtk_print_operation_set_custom_tab_label (GtkPrintOperation  *op,
  * Since: 2.10
  */
 void
-gtk_print_operation_set_pdf_target (GtkPrintOperation *op,
-				    const gchar       *filename)
+gtk_print_operation_set_export_filename (GtkPrintOperation *op,
+					 const gchar       *filename)
 {
   GtkPrintOperationPrivate *priv;
 
@@ -1707,10 +1681,10 @@ gtk_print_operation_set_pdf_target (GtkPrintOperation *op,
 
   priv = op->priv;
 
-  g_free (priv->pdf_target);
-  priv->pdf_target = g_strdup (filename);
+  g_free (priv->export_filename);
+  priv->export_filename = g_strdup (filename);
 
-  g_object_notify (G_OBJECT (op), "pdf-target");
+  g_object_notify (G_OBJECT (op), "export_filename");
 }
 
 /* Creates the initial page setup used for printing unless the
@@ -1766,7 +1740,7 @@ pdf_start_page (GtkPrintOperation *op,
 {
   GtkPaperSize *paper_size;
   cairo_surface_t *surface = op->priv->platform_data;
-  double w, h;
+  gdouble w, h;
 
   paper_size = gtk_page_setup_get_paper_size (page_setup);
 
@@ -1801,25 +1775,28 @@ pdf_end_run (GtkPrintOperation *op,
 static GtkPrintOperationResult
 run_pdf (GtkPrintOperation  *op,
 	 GtkWindow          *parent,
-	 gboolean           *do_print,
-	 GError            **error)
+	 gboolean           *do_print)
 {
   GtkPrintOperationPrivate *priv = op->priv;
   GtkPageSetup *page_setup;
   cairo_surface_t *surface;
   cairo_t *cr;
-  double width, height;
-  /* This will be overwritten later by the non-default size, but
-     we need to pass some size: */
+  gdouble width, height;
+  
+  priv->print_context = _gtk_print_context_new (op);
   
   page_setup = create_page_setup (op);
+  _gtk_print_context_set_page_setup (priv->print_context, page_setup);
+
+  /* This will be overwritten later by the non-default size, but
+     we need to pass some size: */
   width = gtk_page_setup_get_paper_width (page_setup, GTK_UNIT_POINTS);
   height = gtk_page_setup_get_paper_height (page_setup, GTK_UNIT_POINTS);
   g_object_unref (page_setup);
   
-  surface = cairo_pdf_surface_create (priv->pdf_target,
+  surface = cairo_pdf_surface_create (priv->export_filename,
 				      width, height);
-  cairo_pdf_surface_set_dpi (surface, 300, 300);
+  cairo_surface_set_fallback_resolution (surface, 300, 300);
 
   priv->platform_data = surface;
   priv->free_platform_data = (GDestroyNotify) cairo_surface_destroy;
@@ -1863,10 +1840,9 @@ typedef struct
 
   gint page, start, end, inc;
 
-  gboolean initialized;
-
   GtkWidget *progress;
  
+  gboolean initialized;
   gboolean is_preview; 
 } PrintPagesData;
 
@@ -1939,11 +1915,17 @@ print_pages_idle_done (gpointer user_data)
   if (data->progress)
     gtk_widget_destroy (data->progress);
 
-  g_object_unref (data->op);
-
-  if (priv->rloop && !data->is_preview)
+  if (priv->rloop && !data->is_preview) 
     g_main_loop_quit (priv->rloop);
 
+  if (!data->is_preview)
+    g_signal_emit (data->op, signals[DONE], 0,
+		   priv->cancelled ?
+		   GTK_PRINT_OPERATION_RESULT_CANCEL :
+		   GTK_PRINT_OPERATION_RESULT_APPLY);
+  
+  g_object_unref (data->op);
+
   g_free (data);
 
   GDK_THREADS_LEAVE ();
@@ -1979,7 +1961,7 @@ update_progress (PrintPagesData *data)
 
 static void
 common_render_page (GtkPrintOperation *op,
-		    gint page_nr)
+		    gint               page_nr)
 {
   GtkPrintOperationPrivate *priv = op->priv;
   GtkPageSetup *page_setup;
@@ -2192,18 +2174,26 @@ show_progress_timeout (PrintPagesData *data)
 }
 
 static void
-print_pages (GtkPrintOperation *op,
-	     GtkWindow         *parent,
-	     gboolean		is_preview)
+print_pages (GtkPrintOperation       *op,
+	     GtkWindow               *parent,
+	     gboolean                 do_print,
+	     GtkPrintOperationResult  result)
 {
   GtkPrintOperationPrivate *priv = op->priv;
   PrintPagesData *data;
-
+ 
+  if (!do_print) 
+    {
+      _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_FINISHED_ABORTED, NULL);
+      g_signal_emit (op, signals[DONE], 0, result);
+      return;
+  }
+  
   _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_PREPARING, NULL);  
 
   data = g_new0 (PrintPagesData, 1);
   data->op = g_object_ref (op);
-  data->is_preview = is_preview;
+  data->is_preview = (priv->action == GTK_PRINT_OPERATION_ACTION_PREVIEW);
 
   if (priv->show_progress)
     {
@@ -2224,21 +2214,22 @@ print_pages (GtkPrintOperation *op,
       data->progress = progress;
     }
 
-  if (is_preview)
+  if (data->is_preview)
     {
       gboolean handled;
       
       g_signal_emit_by_name (op, "preview",
 			     GTK_PRINT_OPERATION_PREVIEW (op),
-			     op->priv->print_context,
+			     priv->print_context,
 			     parent,
 			     &handled);
       
       if (!handled ||
-	  gtk_print_context_get_cairo_context (priv->print_context) == NULL) {
-	/* Programmer error */
-	g_error ("You must set a cairo context on the print context");
-      }
+	  gtk_print_context_get_cairo_context (priv->print_context) == NULL) 
+	{
+	  /* Programmer error */
+	  g_error ("You must set a cairo context on the print context");
+	}
       
       priv->start_page = preview_start_page;
       priv->end_page = preview_end_page;
@@ -2259,10 +2250,8 @@ print_pages (GtkPrintOperation *op,
 					       print_pages_idle, 
 					       data, 
 					       print_pages_idle_done);
-
-  /* Recursive main loop to make sure we don't exit 
-   * on sync operations 
-   */
+  
+  /* Recursive main loop to make sure we don't exit  on sync operations  */
   if (priv->is_sync)
     {
       priv->rloop = g_main_loop_new (NULL, FALSE);
@@ -2276,9 +2265,34 @@ print_pages (GtkPrintOperation *op,
     }
 }
 
+/**
+ * gtk_print_operation_get_error:
+ * @op: a #GtkPrintOperation
+ * @error: return location for the error
+ * 
+ * Call this when the result of a print operation is
+ * %GTK_PRINT_OPERATION_RESULT_ERROR, either as returned by 
+ * gtk_print_operation_run(), or in the ::done signal handler. 
+ * The returned #GError will contain more details on what went wrong.
+ *
+ * Since: 2.10
+ **/
+void
+gtk_print_operation_get_error (GtkPrintOperation  *op,
+			       GError            **error)
+{
+  g_return_if_fail (GTK_IS_PRINT_OPERATION (op));
+  
+  g_propagate_error (error, op->priv->error);
+
+  op->priv->error = NULL;
+}
+
+
 /**
  * gtk_print_operation_run:
  * @op: a #GtkPrintOperation
+ * @action: the action to start
  * @parent: Transient parent of the dialog, or %NULL
  * @error: Return location for errors, or %NULL
  * 
@@ -2286,11 +2300,16 @@ print_pages (GtkPrintOperation *op,
  * print settings in the print dialog, and then print the
  * document.
  *
- * Note that this function does not return until the rendering of all 
+ * Normally that this function does not return until the rendering of all 
  * pages is complete. You can connect to the ::status-changed signal on
  * @op to obtain some information about the progress of the print operation. 
  * Furthermore, it may use a recursive mainloop to show the print dialog.
- * See gtk_print_operation_run_async() if this is a problem.
+ *
+ * If you call gtk_print_operation_set_allow_async() or set the allow-async
+ * property the operation will run asyncronously if this is supported on the
+ * platform. The ::done signal will be emitted with the operation results when
+ * the operation is done (i.e. when the dialog is canceled, or when the print
+ * succeeds or fails).
  *
  * <informalexample><programlisting>
  * if (settings != NULL)
@@ -2304,7 +2323,7 @@ print_pages (GtkPrintOperation *op,
  * g_signal_connect (print, "draw-page", 
  *                   G_CALLBACK (draw_page), &amp;data);
  *  
- * res = gtk_print_operation_run (print, parent, &amp;error);
+ * res = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, parent, &amp;error);
  *  
  * if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
  *  {
@@ -2331,86 +2350,80 @@ print_pages (GtkPrintOperation *op,
  *   %GTK_PRINT_OPERATION_RESULT_APPLY indicates that the printing was
  *   completed successfully. In this case, it is a good idea to obtain 
  *   the used print settings with gtk_print_operation_get_print_settings() 
- *   and store them for reuse with the next print operation.
+ *   and store them for reuse with the next print operation. A value of
+ *   %GTK_PRINT_OPERATION_RESULT_IN_PROGRESS means the operation is running
+ *   asynchronously, and will emit the ::done signal when done.
  *
  * Since: 2.10
  **/
 GtkPrintOperationResult
-gtk_print_operation_run (GtkPrintOperation  *op,
-			 GtkWindow          *parent,
-			 GError            **error)
+gtk_print_operation_run (GtkPrintOperation        *op,
+			 GtkPrintOperationAction   action,
+			 GtkWindow                *parent,
+			 GError                  **error)
 {
   GtkPrintOperationPrivate *priv;
   GtkPrintOperationResult result;
+  GtkPageSetup *page_setup;
   gboolean do_print;
   
   g_return_val_if_fail (GTK_IS_PRINT_OPERATION (op), 
                         GTK_PRINT_OPERATION_RESULT_ERROR);
 
   priv = op->priv;
-  priv->is_sync = TRUE;
 
-  if (priv->pdf_target != NULL)
-    result = run_pdf (op, parent, &do_print, error);
+  do_print = FALSE;
+  priv->error = NULL;
+  priv->action = action;
+
+  if (priv->print_settings == NULL)
+    priv->print_settings = gtk_print_settings_new ();
+  
+  if (action == GTK_PRINT_OPERATION_ACTION_EXPORT)
+    {
+      priv->is_sync = TRUE;
+      g_return_val_if_fail (priv->export_filename != NULL, GTK_PRINT_OPERATION_RESULT_ERROR);
+      result = run_pdf (op, parent, &do_print);
+    }
+  else if (action == GTK_PRINT_OPERATION_ACTION_PREVIEW)
+    {
+      priv->is_sync = !priv->allow_async;
+      priv->print_context = _gtk_print_context_new (op);
+      page_setup = create_page_setup (op);
+      _gtk_print_context_set_page_setup (priv->print_context, page_setup);
+      g_object_unref (page_setup);
+      do_print = TRUE;
+      result = priv->is_sync ? GTK_PRINT_OPERATION_RESULT_APPLY : GTK_PRINT_OPERATION_RESULT_IN_PROGRESS;
+    }
+#ifndef G_OS_WIN32
+  else if (priv->allow_async)
+    {
+      priv->is_sync = FALSE;
+      _gtk_print_operation_platform_backend_run_dialog_async (op,
+							      action == GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+							      parent,
+							      print_pages);
+      result = GTK_PRINT_OPERATION_RESULT_IN_PROGRESS;
+    }
+#endif
   else
-    result = _gtk_print_operation_platform_backend_run_dialog (op, 
-							       parent,
-							       &do_print,
-							       error);
-  if (do_print)
-    print_pages (op, parent, result == GTK_PRINT_OPERATION_RESULT_PREVIEW);
-  else 
-    _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_FINISHED_ABORTED, NULL);
+    {
+      priv->is_sync = TRUE;
+      result = _gtk_print_operation_platform_backend_run_dialog (op, 
+								 action == GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
+								 parent,
+								 &do_print);
+    }
 
+  if (result != GTK_PRINT_OPERATION_RESULT_IN_PROGRESS)
+    print_pages (op, parent, do_print, result);
+
+  if (priv->error && error)
+    *error = g_error_copy (priv->error);
+  
   return result;
 }
 
-/**
- * gtk_print_operation_run_async:
- * @op: a #GtkPrintOperation
- * @parent: Transient parent of the dialog, or %NULL
- * 
- * Runs the print operation, by first letting the user modify
- * print settings in the print dialog, and then print the
- * document.
- *
- * In contrast to gtk_print_operation_run(), this function returns after 
- * showing the print dialog on platforms that support this, and handles 
- * the printing by connecting a signal handler to the ::response signal 
- * of the dialog. 
- * 
- * If you use this function, it is recommended that you store the modified
- * #GtkPrintSettings in a ::begin-print or ::end-print signal handler.
- * 
- * Since: 2.10
- **/
-void
-gtk_print_operation_run_async (GtkPrintOperation *op,
-			       GtkWindow         *parent)
-{
-  GtkPrintOperationPrivate *priv;
-  gboolean do_print;
-
-  g_return_if_fail (GTK_IS_PRINT_OPERATION (op)); 
-
-  priv = op->priv;
-  priv->is_sync = FALSE;
-
-  if (priv->pdf_target != NULL)
-    {
-      run_pdf (op, parent, &do_print, NULL);
-      if (do_print)
-	print_pages (op, parent, FALSE);
-      else 
-	_gtk_print_operation_set_status (op, GTK_PRINT_STATUS_FINISHED_ABORTED, NULL);
-    }
-  else
-    _gtk_print_operation_platform_backend_run_dialog_async (op, 
-							    parent,
-							    print_pages);
-}
-
-
 /**
  * gtk_print_operation_cancel:
  * @op: a #GtkPrintOperation
